﻿/*
Copyright (c) 2013 Rion Carter

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using System.Text;

using RestSharp;
using Newtonsoft.Json.Linq;
using RestSharp.Contrib;

namespace Rubion.OwnCloud {
    class OcClient {
        RestClient Client;
        string BaseUrl;
        string Username;
        string Password;

        /// <summary>
        /// Initialize an OwnCloud Client object instance directed at a specific OwnCloud instance
        /// </summary>
        /// <param name="baseUrl">Should be a URI like https://somegood.com/cloud</param>
        public OcClient(string baseUrl, string username, string password) {
            BaseUrl = baseUrl;
            Username = username;
            Password = password;

            // Setup the RestClient with cookie container and path
            Client = new RestClient(baseUrl);
            Client.CookieContainer = new System.Net.CookieContainer();

            // Authenticate Part 1 (get base cookie)
            RestRequest Request = new RestRequest();
            Request.RequestFormat = DataFormat.Json;
            Request.Method = Method.GET;
            var resp = Client.Execute(Request);

            // Authenticate Part 2 (get rest of cookies)
            Request = new RestRequest("//", Method.POST);
            Request.AddParameter("user", Username);
            Request.AddParameter("password", Password);
            Request.AddParameter("password-clone", Password);
            Request.AddParameter("remember_login", 1);
            Request.AddParameter("timezone-offset", -7);
            var response = Client.Execute(Request);

            /*
             * At this point we are authenticated to the owncloud instance and make API calls
             */
        }

        #region Files API
         void ListFiles() {
            throw new NotImplementedException("Hasn't been written yet");
        }
         void GetFile() {
            throw new NotImplementedException("Hasn't been written yet");
        }
         void UploadFile() {
            throw new NotImplementedException("Hasn't been written yet");
        }
        #endregion


        #region Music API
        public JObject GetAvailableMusic() {
            // Get collection
            RestRequest Request = new RestRequest("/index.php/apps/media/ajax/api.php?action=get_collection");
            Request.RequestFormat = DataFormat.Json;
            var Response = Client.Execute(Request);
            JObject MusicCollection = JObject.Parse(Response.Content);
            return MusicCollection;
        }

        public byte[] GetAudioFile(string audioStub){
            // Get an appropriately formatted url to the audio file
            string AudioStub_encoded = HttpUtility.UrlEncode(audioStub);
            string AudioFileUrl = "/index.php/apps/media/ajax/api.php?action=play&path=" + AudioStub_encoded;

            // Setup a request to download the file
            RestRequest Request = new RestRequest(AudioFileUrl);

            // Download & return the Audio File
            var Response = Client.Execute(Request);
            return Response.RawBytes;

        }
        #endregion
    }
}
